var BraintreeCheckout = function() {
	this.form = jQuery('#braintree_custom_form').closest('form');
	this.clientToken = this.clientToken = document
			.getElementById('client_token') ? document
			.getElementById('client_token').value : null;
	this.options = braintree_custom_form_options;
	this.add_paymentmethod_page = braintree_custom_form_options.add_paymentmethod_page === 'true' ? true : false;
	this.cart_contains_subscription = braintree_custom_form_options.cart_contains_subscription === 'true' ? true : false;
	this.save_payment_method_enabled = braintree_custom_form_options.save_payment_methods === 'true' ? true : false;
	this.change_payment_method_page = braintree_custom_form_options.change_payment_method === 'true' ? true : false;
	this.setupEvents();
}

var braintreeCheckout = null;

BraintreeCheckout.instance = function() {
	if(braintreeCheckout == null){
		braintreeCheckout = new BraintreeCheckout();
	}
	if (!braintreeCheckout.hasSavedPaymentMethods()) {
		braintreeCheckout.initializeForm();
	} else {
		braintreeCheckout.selectDefaultMethod();
	}
}

BraintreeCheckout.prototype.setupEvents = function() {
	jQuery(this.form).on('checkout_place_order', this.checkoutPlaceOrder);
	jQuery(document.body).on('checkout_error', this.checkoutError);
	jQuery(document.body).on('click', '#cancel_add_new', this.cancelAddNew);
	jQuery(document.body).on('click', '#add_new_method', this.addNewMethod);
	jQuery(document.body).on('click', '.payment-method-item',
			this.paymentMethodSelected);
	
}

BraintreeCheckout.prototype.initializeForm = function() {
	if (braintreeCheckout.initializeCalled) {
		braintreeCheckout.teardown();
	}
	
	jQuery(document.body).trigger('braintree_before_form_initialize');
	braintreeCheckout.initializeCalled = true;
	braintreeCheckout.removeExtraFrames();
	braintree.client.create({
		authorization : this.clientToken
	}, function(err, clientInstance) {
		if (err) {
			//console.log(err);
			return;
		}
		braintree.hostedFields.create({
			client : clientInstance,
			styles : braintreeCheckout.getStyles(),
			fields : braintreeCheckout.getFields(),
		}, function(err, hostedFieldsInstance) {
			if (err) {
				//console.log(err);
				return;
			}
			var submitButton = jQuery(braintreeCheckout.form).find('#place_order')[0];
			braintreeCheckout.removeExtraFrames();
			braintreeCheckout.hostedFieldsInstance = hostedFieldsInstance;
			hostedFieldsInstance.on('validityChange', braintreeCheckout.validityChange);
			hostedFieldsInstance.on('cardTypeChange', braintreeCheckout.cardTypeChange);
			hostedFieldsInstance.on('empty', braintreeCheckout.fieldEmpty);
			hostedFieldsInstance.on('notEmpty', braintreeCheckout.fieldNotEmpty);
			hostedFieldsInstance.on('blur', braintreeCheckout.fieldBlur);
			hostedFieldsInstance.on('focus', braintreeCheckout.fieldFocus);
			hostedFieldsInstance.on('inputSubmitRequest', braintreeCheckout.inputSubmitRequest);
			submitButton.addEventListener('click', function(e) {
				if(braintreeCheckout.isGatewaySelected() && ! braintreeCheckout.isPaymentMethodSelected()){
					e.preventDefault();
					jQuery(submitButton).prop('disabled', true);
					try{
						hostedFieldsInstance.tokenize(function(err, payload) {
							if (err) {
								jQuery(submitButton).prop('disabled', false);
								braintreeCheckout.tokenizationStarted = false;
								jQuery(document.body).trigger('braintree_tokenization_error', err);
								return;
							}
							jQuery(submitButton).prop('disabled', false);
							braintreeCheckout.onPaymentMethodReceived(payload);
						});
					}catch(err){
						
					}
				}
			});
		});
	});
	braintreeCheckout.removeExtraFrames();
}

BraintreeCheckout.prototype.onPaymentMethodReceived = function(response) {
	var element = document.getElementById('payment_method_nonce');
	if (element != null) {
		element.value = response.nonce;
	} else {
		element = document.createElement('input');
		element.type = 'hidden';
		element.name = 'payment_method_nonce';
		element.id = 'payment_method_nonce';
		element.value = response.nonce;
		jQuery(braintreeCheckout.form).append(element);
	}
	braintreeCheckout.paymentMethodReceived = true;
	jQuery(braintreeCheckout.form).submit();
}

BraintreeCheckout.prototype.fieldEmpty = function(event){
	jQuery(document.body).trigger('braintree_field_empty', event);
}

BraintreeCheckout.prototype.fieldNotEmpty = function(event){
	jQuery(document.body).trigger('braintree_field_not_empty', event);
}

BraintreeCheckout.prototype.fieldBlur = function(event){
	jQuery(document.body).trigger('braintree_field_blur', event);
}

BraintreeCheckout.prototype.fieldFocus = function(event){
	jQuery(document.body).trigger('braintree_field_focus', event);
}

BraintreeCheckout.prototype.validityChange = function(event) {
	jQuery(document.body).trigger('braintree_field_validity_change', event);
}

BraintreeCheckout.prototype.cardTypeChange = function(event){
	jQuery(document.body).trigger('braintree_card_type_change', event);
}

BraintreeCheckout.prototype.inputSubmitRequest = function(event){
	jQuery(document.body).trigger('braintree_card_input_submit_request', event);
}

BraintreeCheckout.prototype.checkoutPlaceOrder = function() {
	if (braintreeCheckout.isGatewaySelected()) {
		if (braintreeCheckout.isPaymentMethodSelected()) {
			return true;
		} else {
			if (braintreeCheckout.paymentMethodReceived) {
				return true;
			} else {
				return false;
			}
		}
	}
}

BraintreeCheckout.prototype.isGatewaySelected = function() {
	return document.getElementById('payment_method_braintree_payment_gateway').checked;
}

BraintreeCheckout.prototype.getStyles = function() {
	var styles = braintree_custom_form_options.custom_form_styles;
	styles = JSON.parse(styles);
	return styles;
}

BraintreeCheckout.prototype.getFieldContainerClass = function() {
	return braintree_custom_form_options.custom_form_field_container_class;
}

BraintreeCheckout.prototype.getFields = function() {
	if (!braintreeCheckout.fields) {
		var fields = {
			number : {
				selector : '#card-number',
				placeholder : braintree_custom_form_options.card_number_placeholder
			},
		}
		if(jQuery('#cvv').length){
			fields.cvv = {
				selector : '#cvv',
				placeholder : braintree_custom_form_options.card_cvv_placeholder
			}
		}
		if(jQuery('#postal-code').length){
			fields.postalCode = {
				selector : '#postal-code',
				placeholder : braintree_custom_form_options.card_postal_placeholder
			}
		}
		if (jQuery('#expiration-date').length) {
			fields.expirationDate = {
				selector : '#expiration-date',
				placeholder : braintree_custom_form_options.card_expiration_date_placeholder
			}
		} else if(jQuery('#expiration-month').length) {
			fields.expirationMonth = {
				selector : '#expiration-month',
				placeholder : braintree_custom_form_options.card_expiration_month_placeholder
			};
			fields.expirationYear = {
				selector : '#expiration-year',
				placeholder : braintree_custom_form_options.card_expiration_year_placeholder
			}
		}
		braintreeCheckout.fields = fields;
	}
	return braintreeCheckout.fields;
}

BraintreeCheckout.prototype.teardown = function(){
	try {
		braintreeCheckout.hostedFieldsInstance.teardown(function(callback) {
			braintreeCheckout.initializeCalled = false;
		});
	} catch (err) {
		
	}
}

BraintreeCheckout.prototype.cancelAddNew = function() {
	braintreeCheckout.teardown();
	braintreeCheckout.hideCustomForm();
	braintreeCheckout.selectDefaultMethod();
	braintreeCheckout.showPaymentMethods();
}

BraintreeCheckout.prototype.addNewMethod = function() {
	braintreeCheckout.removeSelectedPaymentMethod();
	braintreeCheckout.hidePaymentMethods();
	braintreeCheckout.initializeForm();
	braintreeCheckout.showCustomForm();
}

BraintreeCheckout.prototype.showCustomForm = function() {
	jQuery('#braintree_custom_form').slideDown(400);
}

BraintreeCheckout.prototype.hideCustomForm = function() {
	jQuery('#braintree_custom_form').slideUp(400);
}

BraintreeCheckout.prototype.hidePaymentMethods = function() {
	jQuery('#saved_payment_methods').slideUp(400);
}

BraintreeCheckout.prototype.showPaymentMethods = function() {
	jQuery('#saved_payment_methods').slideDown(400);
}

BraintreeCheckout.prototype.removeSelectedPaymentMethod = function() {
	jQuery('#selected_payment_method').val("");
}

BraintreeCheckout.prototype.paymentMethodSelected = function() {
	jQuery('.payment-method-item').each(function() {
		jQuery(this).removeClass('selected');
	});
	jQuery('#selected_payment_method').val(jQuery(this).attr('payment-token'));
	jQuery(this).addClass('selected');
}

BraintreeCheckout.prototype.isChangePaymentRequest = function() {
	return jQuery('#order_review').length > 0
			&& !document.getElementById('checkout');
}

BraintreeCheckout.prototype.hasSavedPaymentMethods = function() {
	return jQuery('#saved_payment_methods').length > 0;
}

BraintreeCheckout.prototype.selectDefaultMethod = function() {
	jQuery('.payment-method-item').each(
			function() {
				if (jQuery(this).hasClass('selected')) {
					jQuery('#selected_payment_method').val(
							jQuery(this).attr('payment-token'));
				}
			});
}

BraintreeCheckout.prototype.isPaymentMethodSelected = function() {
	if (jQuery('#selected_payment_method').length > 0
			&& jQuery('#selected_payment_method').val() !== "") {
		return true;
	}
	return false;
}

BraintreeCheckout.prototype.hideSavedPaymentMethodCheckbox = function(){
	jQuery('.save-card-info').hide();
}

BraintreeCheckout.prototype.removeExtraFrames = function() {
	var fields = braintreeCheckout.getFields();
	for ( var key in fields) {
		var field = fields[key];
		jQuery.each(jQuery(field.selector).children('iFrame'), function(index) {
			if (index > 0) {
				jQuery(this).remove();
			}
		})
	}
}

BraintreeCheckout.verifyFormInitialized = function(){
	if(braintreeCheckout == null || !braintreeCheckout.initializeCalled){
		jQuery(document.body).trigger('updated_checkout');
	}
}

BraintreeCheckout.prototype.checkoutError = function() {
	braintreeCheckout.paymentMethodReceived = false;
}

jQuery(document.body).on('updated_checkout',
		BraintreeCheckout.instance);

setTimeout(BraintreeCheckout.verifyFormInitialized, 3000);