<?php
if (! defined ( 'ABSPATH' )) {
	exit (); // Exit if accessed directly
}
/**
 * PayPal specific class to be used when merchants are using a custom payment form.
 * PayPal
 * will be treated as a separate gateway.
 *
 * @author Clayton Rogers
 * @since 2.4.3
 * @version 2.4.3
 *         
 */
class WC_PayPal_Payments extends WC_Braintree_Payments {
	public $supports = array (
			'subscriptions',
			'products',
			'subscription_cancellation',
			'multiple_subscriptions',
			'subscription_amount_changes',
			'subscription_date_changes',
			'default_credit_card_form',
			'refunds',
			'pre-orders',
			'subscription_payment_method_change_admin',
			'subscription_reactivation',
			'subscription_suspension',
			'subscription_payment_method_change_customer' 
	);
	public static function init() {
		add_filter ( 'woocommerce_payment_gateways', __CLASS__ . '::add_gateway' );
		parent::init ();
	}
	public function __construct() {
		$this->enabled = self::is_gateway_active () ? 'yes' : 'no';
		$this->id = self::paypal_gateway_name;
		$this->title = BT_Manager ()->get_option ( 'paypal_gateway_title' );
		$this->has_fields = true;
		$this->method_title = __ ( 'PayPal Braintree Payment Gateay', 'braintree' );
		add_action ( 'wp_enqueue_scripts', __CLASS__ . '::load_scripts' );
	}
	public static function is_gateway_active() {
		return (BT_Manager ()->isActive ( 'custom_form' ) && BT_Manager ()->isActive ( 'enable_paypal' )) || BT_Manager ()->isActive ( 'paypal_only' );
	}
	public static function load_scripts() {
		if (BT_Manager ()->isActive ( 'enable_paypal' ) || BT_Manager ()->isActive ( 'paypal_only' )) {
			/*
			 * wp_enqueue_script ( 'paypal-braintree-checkout', WC_BRAINTREE_ASSETS . 'js/paypal-checkout.js', array (
			 * 'wc-checkout'
			 * ), null, true );
			 */
			wp_enqueue_script ( 'paypal-component', 'https://js.braintreegateway.com/web/3.0.0-beta.10/js/paypal.min.js', array (
					'wc-checkout' 
			), null, true );
			wp_enqueue_script ( 'paypal-braintree-checkout', WC_BRAINTREE_ASSETS . 'js/paypal-checkout.js', array (
					'wc-checkout' 
			), null, true );
			$paypal_design = BT_Manager ()->get_paypal_design ();
			wp_enqueue_style ( 'paypal-design-css', $paypal_design ['css'] );
			if (BT_Manager ()->isActive ( 'paypal_only' )) {
				wp_enqueue_script ( 'braintree-V3-component', 'https://js.braintreegateway.com/web/3.0.0-beta.10/js/client.min.js', array (
						is_checkout () ? 'wc-checkout' : 'jquery' 
				), null, true );
			}
		}
	}
	
	/**
	 * Display the PayPal specific payment fields.
	 *
	 * {@inheritDoc}
	 *
	 * @see WC_Braintree_Payments::payment_fields()
	 */
	public function payment_fields() {
		if (is_checkout ()) {
			include WC_BRAINTREE_PLUGIN . 'payments/forms/paypal/paypal-form.php';
		}
	}
	
	/**
	 * Add the PayPal payment gateway to the array of methods.
	 *
	 * @param array $methods        	
	 */
	public static function add_gateway($methods = array()) {
		if ((BT_Manager ()->isActive ( 'custom_form' ) && BT_Manager ()->isActive ( 'enable_paypal' )) || BT_Manager ()->isActive ( 'paypal_only' )) {
			$methods [] = 'WC_PayPal_Payments';
		}
		return $methods;
	}
}
WC_PayPal_Payments::init ();